/*
 *  Copyright 2011 Ruediger Gad
 *
 *  This file is part of Q To-Do.
 *
 *  Q To-Do is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Q To-Do is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Q To-Do.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  ###################
 *
 *  Note: The code of the TreeView is additionally released under the terms
 *  of the GNU Lesser General Public License (LGPL) as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  Files considered part of the TreeView are: TreeView.qml, NodeListView.qml,
 *  and nodelisthelper.js. These files are additionally licensed under the
 *  terms of the LGPL.
 *
 */

import QtQuick 1.1

Item{
    id: nodeListItem

    property alias model: nodeListView.model
    property alias currentIndex: nodeListView.currentIndex
    property alias currentItem: nodeListView.currentItem

    signal countChanged(int count)
    signal expandTree()
    signal leafNodeSelected()

    function updateLabels(){
        var listEmpty = (nodeListView.model.rowCount() <= 0)
        emptyListRectangle.visible = listEmpty
    }

    onExpandTree: {
        treeView.addView(nodeListItem)
        treeView.updateSubView(model, currentIndex)
    }

    onLeafNodeSelected: {
        treeView.clearSubLists()
    }

    Rectangle{
        id: emptyListRectangle
        anchors.fill: parent

        Text {
            id: noContentLabel
            text: "No entries yet"
            font.pixelSize: 60; anchors.bottom: explanationLabel.top; anchors.bottomMargin: 50; anchors.horizontalCenter: parent.horizontalCenter; color: "lightgray"
        }

        Text {
            id: explanationLabel
            text: "Use + to add entries."
            font.pixelSize: 40;  color: "lightgray"; anchors.centerIn: parent
        }
    }

    PinchArea{
        id: dndArea
        anchors.fill: parent

        property real lastPosition: 0
        property real moveDelta: 40

        onPinchStarted: lastPosition = pinch.startPoint2.y

        onPinchUpdated: {
            var currentPosition = pinch.point2.y

            if(currentPosition === pinch.point1.y)
                return

            if(lastPosition - currentPosition > moveDelta){
                lastPosition = currentPosition
                moveItem(nodeListView.currentIndex - 1)
            }else if (lastPosition - currentPosition < -moveDelta){
                lastPosition = currentPosition
                moveItem(nodeListView.currentIndex + 1)
            }
        }

        onPinchFinished: storage.save()

        function moveItem(targetIndex){
            if(targetIndex >= 0 && targetIndex < nodeListView.count){
                nodeListView.model.move(nodeListView.currentIndex, targetIndex, 1)
                nodeListView.currentIndex = targetIndex
            }
        }
    }

    ListView{
        id: nodeListView
        anchors.fill: parent

        onCountChanged: {
            nodeListItem.countChanged(count)
            updateLabels()
        }

        onCurrentIndexChanged: {
            /*
             * Only update the TreeView current item and index if there had actually a
             * valid item been selected. An index of -1 is used to clear the selection
             * of the ListView from the TreeView. Hence, we do not want this change
             * to propagate back to the TreeView.
             */
            if(currentIndex >= 0){
                treeView.currentItem = currentItem
                treeView.currentIndex = currentIndex
                updateLabels()
            }
        }

        delegate: Rectangle{
            id: delegate
            width: parent.width
            height: textDelegate.height

            function selectItem() {
                currentIndex = index
                if(isExpandable){
                    expandTree()
                }else{
                    leafNodeSelected()
                }
            }

            /*
             * These properties are used to access the item properties/data of
             * the current item as returned via the ListView currentItem property.
             * Customize this as needed for your purpose.
             */
            property string text: elementText
            property string type: tagName
            property string color: elementColor
            property bool done: isDone

            /*
             * Begin of custom code to display the data. Here the Q To-Do to-do or
             * note elements are shown. Customize this to display your own stuff.
             */
            Rectangle {
                id: contentDelegate
                anchors.left: parent.left
                anchors.right: nextButton.left
                height: textDelegate.height

                Image {
                    id: elementIcon
                    height: textDelegate.height
                    fillMode: Image.PreserveAspectFit
                    source: (tagName === "to-do")
                            ? "../icons/to-do_" + (isDone ? "done_" : "") + elementColor + ".svg"
                            : "../icons/note.svg"
                }

                Text {
                    id: textDelegate
                    anchors.left: elementIcon.right
                    anchors.leftMargin: 10
                    anchors.right: parent.right

                    text: elementText;
                    font.pixelSize: 42
                    horizontalAlignment: Text.AlignHLeft
                    wrapMode: Text.WordWrap
                    color: contentMouseArea.pressed ? "#78bfff" : "black"
                }

                MouseArea{
                    id: contentMouseArea
                    anchors.fill: parent
                    onClicked: selectItem()
                }
            }
            /*
             * End of custom code for displaying the data.
             */

            Rectangle {
                id: nextButton
                anchors.right: parent.right
                anchors.rightMargin: 4
                width: (! isExpandable) ? 0 : 40
                height: contentDelegate.height

                Rectangle{
                    id: button
                    anchors.centerIn: parent
                    width: 40
                    height: 40

                    visible: isExpandable

                    radius: 5
                    border{ color: "gray"; width: 3 }
                    color: nextMouseArea.pressed ? "gray" : "lightgray"

                    Image {
                        id: nextIcon
                        anchors.fill: parent
                        fillMode: Image.PreserveAspectFit

                        opacity: (isLeaf) ? 0.3 : 1
                        source: "/usr/share/themes/blanco/meegotouch/icons/icon-m-toolbar-next.png"
                    }

                    MouseArea{
                        id: nextMouseArea
                        anchors.fill: parent

                        onClicked: {
                            selectItem()
                            treeView.currentLevel++
                        }
                    }
                }
            }
        }

        highlightMoveDuration: 200
        highlight: Rectangle {
            id: highlightRectangle
            color: "#78bfff"
            width: parent.width
            /*
             * Set z to a seemingly insane high value. For some strange/unknown reason the
             * the highlight is not shown at all with z=0. For z=1 the highligh is shown but
             * disappears for parts that had been at least once out of the screen bounds
             * (i.e. had not been visible in the list). Hence, just in case set z=32.
             */
            z:32
            opacity: 0.4
        }
    }
}
